@@ -4,6 +4,7 @@
  * For license terms, see the file COPYING in this directory.
  */
 #include "config.h"
+#include "util.h"
 #include "fetchmail.h"
 
 #include <stdio.h>
@@ -28,7 +29,6 @@
 
 #include <langinfo.h>
 
-#include "socket.h"
 #include "tunable.h"
 #include "smtp.h"
 #include "netrc.h"
@@ -478,7 +478,7 @@ int main(int argc, char **argv)
     /* parse the ~/.netrc file (if present) for future password lookups. */
     netrc_file = prependdir (NETRC_FILE, home);
     netrc_entry *netrc_list;
-    netrc_list = parse_netrc(netrc_file);
+    netrc_list = netrc_parse(netrc_file);
     netrc_parsetime = time(NULL);
     { // do not put this under netrc_list - if .netrc were empty, netrc_list would be NULL.
         // note the parse time of netrc
@@ -503,7 +503,7 @@ int main(int argc, char **argv)
 		const netrc_entry *p;
 
 		/* look up the pollname and account in the .netrc file. */
-		p = search_netrc(netrc_list,
+		p = netrc_search(netrc_list,
 				 ctl->server.pollname, ctl->remotename);
 		/* if we find a matching entry with a password, use it */
 		if (p && p->password)
@@ -513,13 +513,30 @@ int main(int argc, char **argv)
 		/* otherwise try with "via" name if there is one */
 		else if (ctl->server.via)
 		{
-		    p = search_netrc(netrc_list, 
+		    p = netrc_search(netrc_list,
 				     ctl->server.via, ctl->remotename);
 		    if (p && p->password)
 		        ctl->password = xstrdup(p->password);
 		}
 	    }
 	}
+        if (ctl->active && !(implicitmode && ctl->server.skip) && ctl->server.esmtp_name && !ctl->server.esmtp_password)
+        {
+                // get ESMTP passwords from .netrc - the user names need to be in .fetchmailrc
+                for (struct idlist *i = ctl->smtphunt; i; i = i->next) {
+                        char *instring = xstrdup(i->id);
+                        struct sink_host_st sinkdata = sink_parse_host(instring);
+                        const netrc_entry *entry = netrc_search(netrc_list, sinkdata.identifier, ctl->server.esmtp_name);
+                        if (!entry)
+                                entry = netrc_search(netrc_list, sinkdata.servername, ctl->server.esmtp_name);
+                        if (entry && entry->password) {
+                                ctl->server.esmtp_password = xstrdup(entry->password);
+                                xfree(instring);
+                                break;
+                        }
+                        xfree(instring);
+                }
+        }
 	if (ctl->moveto != NULL && ctl->server.protocol != P_IMAP)
 	{
 	    (void) fprintf(stderr,
@@ -528,7 +545,7 @@ int main(int argc, char **argv)
 	}
     }
 
-    free_netrc(netrc_list);
+    netrc_free(netrc_list);
     netrc_list = NULL;
 
     /* perhaps we just want to check options? */
@@ -778,15 +795,15 @@ int main(int argc, char **argv)
     do {
 	/* 
 	 * Check to see if the rcfile has been touched.  If so,
-	 * re-exec so the file will be reread.  Doing it this way
+	 * re-exec so the file will be reread.	Doing it this way
 	 * avoids all the complications of trying to deallocate the
 	 * in-core control structures -- and the potential memory
 	 * leaks...
 	 */
 	struct stat	rcstat;
-        struct stat	netrcstat;
-        memset(&rcstat, 0, sizeof rcstat);
-        memset(&netrcstat, 0, sizeof netrcstat);
+	struct stat	netrcstat;
+	memset(&rcstat, 0, sizeof rcstat);
+	memset(&netrcstat, 0, sizeof netrcstat);
 
 	if (strcmp(rcfile, "-") == 0) {
 	    /* do nothing */
@@ -796,11 +813,11 @@ int main(int argc, char **argv)
 		       GT_("couldn't time-check %s (error %d)\n"),
 		       rcfile, errno);
 	}
-        if (-1 == stat(netrc_file, &netrcstat) && ENOENT != errno) {
-                report(stderr, 
-                        GT_("couldn't time-check %s (error %d)\n"),
-                        netrc_file, errno);
-        }
+	if (-1 == stat(netrc_file, &netrcstat) && ENOENT != errno) {
+		report(stderr, 
+			GT_("couldn't time-check %s (error %d)\n"),
+			netrc_file, errno);
+	}
 	else if (rcstat.st_mtime > parsetime || netrcstat.st_mtime > netrc_parsetime)
 	{
 	    report(stdout, GT_("restarting fetchmail (%s changed)\n"), rcstat.st_mtime > parsetime ? rcfile : netrc_file);
@@ -1415,7 +1432,7 @@ static int load_params(int argc, char **argv, int optind)
 	    /* if `user' doesn't name a real local user, try to run as root */
 	    if ((pw = getpwnam(user)) == (struct passwd *)NULL)
 		ctl->uid = 0;
-            else
+	    else
 		ctl->uid = pw->pw_uid;	/* for local delivery via MDA */
 	    if (!ctl->localnames)	/* for local delivery via SMTP */
 		save_str_pair(&ctl->localnames, user, NULL);
@@ -2077,7 +2094,7 @@ static void dump_params (struct runctl *runp,
 			       : GT_("  DNS lookup for multidrop addresses is disabled.\n"));
 			if (ctl->server.dns)
 			{
-	       		    if (ctl->server.checkalias)
+			    if (ctl->server.checkalias)
 				printf(GT_("  Server aliases will be compared with multidrop addresses by IP address.\n"));
 			    else
 				printf(GT_("  Server aliases will be compared with multidrop addresses by name.\n"));
