@@ -9,12 +9,13 @@
   description: 	getpass() replacement which allows for long passwords.
                 This version hacked by Wilfred Teiken, allowing the
                 password to be piped to fetchmail.
- 
+
  ***********************************************************************/
 
 #include "config.h"
 #include "fetchmail.h"
 
+#include <errno.h>
 #include <stdio.h>
 #include <signal.h>
 #include <fcntl.h>
@@ -36,37 +37,38 @@ static void disable_tty_echo(void);
 static void restore_tty_state(void);
 static void sigint_handler(int);
 
-char *fm_getpassword(char *prompt)
+static char pbuf[INPUT_BUF_SIZE];
+
+void fm_clearpassword(void)
+{
+    fm_safe_clearmem(pbuf, sizeof(pbuf));
+}
+
+char *fm_getpassword(const char *prompt)
 {
     char *p;
-    int c;
-    FILE *fi;
-    static char pbuf[INPUT_BUF_SIZE];
+    FILE *fi = NULL;
     SIGHANDLERTYPE sig = 0;	/* initialization pacifies -Wall */
 
     int istty = isatty(0);
+    ttyfd = STDIN_FILENO;
 
     /* get the file descriptor for the actual input device if it's a tty */
     if (istty)
     {
-        int fd = open("/dev/tty", O_RDWR);
-	if (-1 == fd || !(fi = fdopen(fd, "r")))
-	    fi = stdin;
-	else
+        fi = fopen("/dev/tty", "r+");
+	if (fi) {
 	    setbuf(fi, (char *)NULL);
-    }
-    else
-	fi = stdin;
+	    /* store descriptor for the controlling tty */
+	    ttyfd = fileno(fi);
+	} else {
+	    report(stderr, "fm_getpassword: fopen(\"/dev/tty\", \"r+\"): %s\n", strerror(errno));
+	}
 
-    /* store descriptor for the tty */
-    ttyfd = fileno(fi);
-
-    if (istty)
-    {
 	/* preserve tty state before turning off echo */
 	save_tty_state();
 
-	/* now that we have the current tty state, we can catch SIGINT and  
+	/* now that we have the current tty state, we can catch SIGINT and
 	   exit gracefully */
 	sig = set_signal_handler(SIGINT, sigint_handler);
 
@@ -74,32 +76,42 @@ char *fm_getpassword(char *prompt)
 	disable_tty_echo();
 
 	/* display the prompt and get the input string */
-	fprintf(stderr, "%s", prompt);
+	fputs(prompt, stderr);
     }
 
-    for (p = pbuf; (c = getc(fi))!='\n' && c!=EOF;)
-    {
-	if (p < &pbuf[INPUT_BUF_SIZE - 1])
-	    *p++ = c;
+    /* regardless of if the input is a TTY or not, read characters
+     * until we have a LF or an EOF, truncating if we reach full
+     * buffer size */
+    FILE *readstream = fi ? fi : stdin;
+    clearerr(readstream);
+    p = fgets(pbuf, sizeof(pbuf), readstream);
+    int save_errno = errno;
+    if (!p && !ferror(readstream)) {
+	// fgets() returns NULL if EOF happens early,
+	// so don't report error in that case
+	p = pbuf;
     }
-    *p = '\0';
-
-    /* write a newline so cursor won't appear to hang */
-    if (fi != stdin)
-	fprintf(stderr, "\n");
+    // Zap LF:
+    (void)strtok(pbuf, "\n");
 
     if (istty)
     {
+	/* write a newline so cursor won't appear to hang */
+	fprintf(stderr, "\n");
+
 	/* restore previous state of the tty */
 	restore_tty_state();
 
 	/* restore previous state of SIGINT */
 	set_signal_handler(SIGINT, sig);
-    }
-    if (fi != stdin)
-	fclose(fi);	/* not checking should be safe, file mode was "r" */
 
-    return(pbuf);
+	if (fi) {
+	    (void)fclose(fi);	/* not checking should be safe, file mode was "r" */
+	}
+    }
+
+    errno = save_errno;
+    return p;
 }
 
 static void save_tty_state (void)
@@ -108,7 +120,7 @@ static void save_tty_state (void)
     flags = termb.c_lflag;
 }
 
-static void disable_tty_echo(void) 
+static void disable_tty_echo(void)
 {
     /* turn off echo on the tty */
     termb.c_lflag &= ~ECHO;
@@ -125,6 +137,7 @@ static void restore_tty_state(void)
 static void sigint_handler(int signum)
 {
     (void)signum;
+    fm_clearpassword();
     restore_tty_state();
     report(stderr, GT_("\nCaught SIGINT... bailing out.\n"));
     exit(1);
