@@ -23,14 +23,15 @@
 /*
  * Note: this code is harmless on little-endian machines.
  */
-static void byteReverse(unsigned char *buf, unsigned longs)
+static void byteReverse(uint32_t *buf, unsigned longs)
 {
     uint32_t t;
     do {
-	t = (uint32_t) ((unsigned) buf[3] << 8 | buf[2]) << 16 |
-	    ((unsigned) buf[1] << 8 | buf[0]);
-	*(uint32_t *) buf = t;
-	buf += 4;
+	const uint8_t *pbuf = (const uint8_t *)buf; // reinterpret
+	t = (uint32_t) ((uint32_t) pbuf[3] << 8u | pbuf[2]) << 16u
+		| ((unsigned) pbuf[1] << 8u | pbuf[0]);
+	*buf = t;
+	++ buf;
     } while (--longs);
 }
 
@@ -78,7 +79,7 @@ void MD5Update(struct MD5Context *ctx, const void *buf_, unsigned len)
 	    return;
 	}
 	memmove(p, buf, t);
-	byteReverse(ctx->u.in, 16);
+	byteReverse(ctx->u.in32, 16);
 	MD5Transform(ctx->buf, ctx->u.in32);
 	buf += t;
 	len -= t;
@@ -87,7 +88,7 @@ void MD5Update(struct MD5Context *ctx, const void *buf_, unsigned len)
 
     while (len >= 64) {
 	memmove(ctx->u.in, buf, 64);
-	byteReverse(ctx->u.in, 16);
+	byteReverse(ctx->u.in32, 16);
 	MD5Transform(ctx->buf, ctx->u.in32);
 	buf += 64;
 	len -= 64;
@@ -122,7 +123,7 @@ void MD5Final(void *digest, struct MD5Context *ctx)
     if (count < 8) {
 	/* Two lots of padding:  Pad the first block to 64 bytes */
 	memset(p, 0, count);
-	byteReverse(ctx->u.in, 16);
+	byteReverse(ctx->u.in32, 16);
 	MD5Transform(ctx->buf, ctx->u.in32);
 
 	/* Now fill the next block with 56 bytes */
@@ -131,14 +132,14 @@ void MD5Final(void *digest, struct MD5Context *ctx)
 	/* Pad block to 56 bytes */
 	memset(p, 0, count - 8);
     }
-    byteReverse(ctx->u.in, 14);
+    byteReverse(ctx->u.in32, 14);
 
     /* Append length in bits and transform */
     ctx->u.in32[14] = ctx->bits[0];
     ctx->u.in32[15] = ctx->bits[1];
 
     MD5Transform(ctx->buf, ctx->u.in32);
-    byteReverse((unsigned char *) ctx->buf, 4);
+    byteReverse(ctx->buf, 4);
     memmove(digest, ctx->buf, 16);
     memset(ctx, 0, sizeof(*ctx));	/* In case it's sensitive */
 }
