@@ -36,17 +36,17 @@ __BEGIN_DECLS
    Diagnostic messages are through report(), except for fopen() errors with ENOENT.
    \return NULL is returned if the file could not be
    parsed.  */
-netrc_entry *parse_netrc (char *file);
+netrc_entry *netrc_parse (char *file);
 
 /** Search the netrc database for an account password.
  * The DEFAULT entry is never matched, it is designed for anonftp auto-logins.
  * \return the \a netrc_entry from \a list corresponding to \a host and user \a account.
  * \return NULL is returned if no such entry exists.
  */
-const netrc_entry *search_netrc (netrc_entry *list, char *host, char *account);
+const netrc_entry *netrc_search (netrc_entry *list, char *host, char *account);
 
 /** Free memory for the entire \a list netrc_entry database */
-void free_netrc (netrc_entry *list);
+void netrc_free (netrc_entry *list);
 __END_DECLS
 
 #endif /* _NETRC_H_ */
