@@ -1,6 +1,5 @@
-/*
-    rfc2047e.c - encode a string as per RFC-2047
-    Copyright (C) 2004  Matthias Andree
+/** \file rfc2047e.c - encode a string as per RFC-2047
+    \copyright Copyright (C) 2004-2024  Matthias Andree
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,21 +26,27 @@
 #include <stdlib.h>
 #include <assert.h>
 
-static const char noenc[] = "!\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";
-static const char encchars[] = "!\"#$%&'*+,-./0123456789:;<>@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^`abcdefghijklmnopqrstuvwxyz{|}~";
-static const char ws[] = " \t\r\n";
+/** which characters are permitted to be on header line words without need of 
+ * RFC-2047 encoding */
+static const char permitted_unencoded_chars[] = "!\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";
+/** which characters are permitted inside RFC-2047 encoded-words */
+static const char permitted_qpencoded_chars[] = "!\"#$%&'*+,-./0123456789:;<>@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^`abcdefghijklmnopqrstuvwxyz{|}~";
+/** list of whitespace characters in RFC-5322 format headers */
+static const char whitespace_in_header_chars[] = " \t\r\n";
 
 #ifdef TEST
 void report (FILE *fp, const char *format, ...) { (void)fp; (void)format;}
 #endif
 
-static int needs_enc(const char *string) {
-    if (strspn(string, noenc) < strlen(string))
-	return 1;
+static bool needs_enc(const char *string)
+/** needs_enc guesses whether \a string needs any form of RFC-2047 encoding. */
+{
+    if (strspn(string, permitted_unencoded_chars) < strlen(string))
+	return true;
     if (strncmp(string, "=?", 2) == 0
 	    && strcmp(string + strlen(string) - 2, "?=") == 0)
-	return 1;
-    return 0;
+	return true;
+    return false;
 }
 
 /** \bug This RFC-2047 encoder is simplistic and violates the RFC-2047
@@ -77,7 +82,7 @@ static char *encode_words(char *const *words, int nwords, const char *charset)
 		t = stpcpy(t, "?Q?");
 	    }
 	    if (*u == ' ') { *t++ = '_'; continue; }
-	    if (strchr(encchars, *u)) { *t++ = *u; continue; }
+	    if (strchr(permitted_qpencoded_chars, *u)) { *t++ = *u; continue; }
 	    sprintf(t, "=%02X", (unsigned int)((unsigned char)*u));
 	    t += 3;
 	}
@@ -110,24 +115,24 @@ char *rfc2047e(const char *string, const char *charset) {
     r = string;
     while (*r) {
 	count++;
-	r += strcspn(r, ws);
+	r += strcspn(r, whitespace_in_header_chars);
 	if (!*r) break;
 	count++;
-	r += strspn(r, ws);
+	r += strspn(r, whitespace_in_header_chars);
     }
     words = (char **)xmalloc(sizeof(char *) * (count + 1));
 
     idx = 0;
     r = string;
     while (*r) {
-	l = strcspn(r, ws);
+	l = strcspn(r, whitespace_in_header_chars);
 	words[idx] = (char *)xmalloc(l+1);
 	memcpy(words[idx], r, l);
 	words[idx][l] = '\0';
 	idx++;
 	r += l;
 	if (!*r) break;
-	l = strspn(r, ws);
+	l = strspn(r, whitespace_in_header_chars);
 	words[idx] = (char *)xmalloc(l+1);
 	memcpy(words[idx], r, l);
 	words[idx][l] = '\0';
