@@ -698,7 +698,7 @@ static int SetRealmService(char *bufp)
 
 static void GenChallenge(unsigned char *buf, int len)
 {
-    int  i;
+    int  i = 0;
     FILE *devrandom;
 
     devrandom = fopen("/dev/urandom","rb");
@@ -711,8 +711,14 @@ static void GenChallenge(unsigned char *buf, int len)
 	report(stdout, GT_("    attacks by a dishonest service are possible.)\n"));
     }
 
-    for(i=0; i<len; i++)
-	buf[i] = devrandom ? fgetc(devrandom) : random();
+    if (devrandom) i = fread(buf, 1, len, devrandom);
+    /* TODO: This is garbage and should use RAND_bytes or arc4random_buf() at
+     * least, but RPA based on MD5 should be phased out anyways, but apparently
+     * the Dovecot software still documents it
+     */
+    // on short reads, fill up with lower-quality random()
+    for(; i<len; i++)
+	buf[i] = random();
 
     if (devrandom)
 	fclose(devrandom);	/* should be safe, file mode was "r" */
