@@ -7,11 +7,13 @@
  * listener daemon or an MDA pipe.
  *
  * Copyright 1998 by Eric S. Raymond
+ * Copyright 2004 - 2025 by Matthias Andree, especially TLS for SMTP
  * For license terms, see the file COPYING in this directory.
  */
 
 #include  "config.h"
 #include  "fetchmail.h"
+#include "util.h"
 
 #include  <stdio.h>
 #include  <errno.h>
@@ -66,8 +68,65 @@ static void smtp_rset(struct query *ctl)
     }
 }
 
+struct sink_host_st sink_parse_host(char *parsed_host /** will be modified through strtok_r */)
+{
+    /** split \a parsed_host (which will be modified) into components -- Syntax:
+     * hostname/{port|service}[/starttls|/tls|/notls][/tlsproto=xxx][/servername=XXX]
+     * \returns a \ref sink_host_st structure by value
+     */
+    struct sink_host_st retst = SINKDATA_INITIALIZER;
+    char *parts[5] = {NULL, NULL, NULL, NULL, NULL};
+    char *remainder = NULL;
+    retst.tlsproto = "auto";
+    size_t partcnt = str_split(parsed_host, "/", parts, SINK_HOSTNAME_PARTS, &remainder);
+    retst.identifier = parts[0];
+    retst.servername = parts[0];
+    if (remainder && *remainder)
+	report(stderr, GT_("WARNING: smtphost contains unparsed option part %s\n"),
+		remainder);
+    if (0 == strcmp(parsed_host, "localhost"))
+	retst.tlsmode = NOTLS; // for localhost, tlsmode defaults to off.
+    else if (0 == strcmp(parsed_host, "::1"))
+        retst.tlsmode = NOTLS;  // ditto for the IPv6 loopback address
+    else if (regex_ere_search("^127\\.(0\\.0\\.|0\\.1\\.)?1$", parsed_host))
+        retst.tlsmode = NOTLS;  // ditto for the common IPv4 loopback address variants, 127.0.0.1, 127.0.1.1, 127.1
+    for (size_t i = 1; i < partcnt; i++) {
+	if (!strcmp("tls", parts[i]))
+	    retst.tlsmode = TLSWRAPPED;
+	else if (!strcmp("starttls", parts[i]))
+	    retst.tlsmode = STARTTLS;
+	else if (!strcmp("notls", parts[i]))
+	    retst.tlsmode = NOTLS;
+	else if (str_startswith(parts[i], "tlsproto=")) {
+	    retst.tlsproto = strchr(parts[i], '=') + 1;
+	} else if (str_startswith(parts[i], "servername=")) {
+	    retst.servername = strchr(parts[i], '=') + 1;
+	} else {
+	    retst.portname = parts[i];
+	}
+    }
+    if (!retst.portname || !*retst.portname) {
+	switch (retst.tlsmode) {
+	    case NOTLS:
+	    case MAYSTARTTLS:
+		retst.portname = SMTP_PORT;
+		break;
+
+	    case STARTTLS:
+		retst.portname = "587";
+		break;
+
+	    case TLSWRAPPED:
+		retst.portname = "465";
+		break;
+	}
+    }
+    return retst;
+}
+
 int smtp_setup(struct query *ctl)
-/* try to open a socket to the appropriate SMTP server for this query */ 
+/** try to open a socket to the appropriate SMTP server for this query,
+ * including opportunistic or mandatory STARTTLS or TLS-wrapped modes */
 {
     /* maybe it's time to close the socket in order to force delivery */
     if (last_smtp_ok > 0 && time((time_t *)NULL) - last_smtp_ok > mytimeout)
@@ -106,6 +165,7 @@ int smtp_setup(struct query *ctl)
 	const char *id_me = run.invisible ? ctl->server.truename : fetchmailhost;
 	int oldphase;
 	char *parsed_host = NULL;
+        bool greeting_consumed;
 
 	errno = 0;
 
@@ -120,8 +180,8 @@ int smtp_setup(struct query *ctl)
 	set_timeout(ctl->server.timeout);
 	for (idp = ctl->smtphunt; idp; idp = idp->next)
 	{
-	    char	*cp;
-	    const char	*portnum = SMTP_PORT;
+            struct sink_host_st sinkdata = SINKDATA_INITIALIZER;
+            greeting_consumed = false;
 
 	    ctl->smtphost = idp->id;  /* remember last host tried. */
 	    if (ctl->smtphost[0]=='/')
@@ -135,21 +195,103 @@ int smtp_setup(struct query *ctl)
 	    {
 		ctl->smtphostmode = ctl->listener;
 		parsed_host = xstrdup(idp->id);
-		if ((cp = strrchr(parsed_host, '/')))
-		{
-		    *cp++ = 0;
-		    if (cp[0])
-			portnum = cp;
-		}
-		if ((ctl->smtp_socket = SockOpen(parsed_host,portnum,
+                sinkdata = sink_parse_host(parsed_host);
+                /* split components -- Syntax: hostname/{port|service}[/starttls|/tls|/notls][/tlsproto=xxx][/servername=XXX] */
+
+#ifndef SSL_ENABLE
+                switch (sinkdata.tlsmode) {
+                case NOTLS:
+                case MAYSTARTTLS:
+                        break;
+                default:
+                        report(stderr, "smtphost %s: %s\n", idp->id, GT_("SSL is not enabled"));
+                        xfree(parsed_host);
+                        continue;
+                }
+#endif
+
+		if ((ctl->smtp_socket = SockOpen(parsed_host, sinkdata.portname,
 				ctl->server.plugout, &ai1)) == -1)
 		{
 		    xfree(parsed_host);
 		    continue;
 		}
-	    }
 
-	    /* return immediately for ODMR */
+                // avoid interspersing normal output with a fingerprint:
+                const char *fingerprintmode = outlevel >= O_VERBOSE ? "{SHA256}*" : NULL;
+                int res = -1000;
+
+                switch (sinkdata.tlsmode) {
+#ifdef SSL_ENABLE
+                case TLSWRAPPED: {
+                  char *remotename = NULL;
+                  int res = SSLOpen(ctl->smtp_socket, NULL, NULL, sinkdata.tlsproto, 1,
+                                    ctl->sslcertfile, ctl->sslcertpath, fingerprintmode,
+                                    sinkdata.servername, parsed_host, &remotename);
+                  if (res) {
+                    report(stderr, "%s: %s\n", idp->id,
+                        GT_("TLS wrapped connection failed.\n"));
+                    smtp_close(ctl, 0);
+                    xfree(parsed_host);
+                    continue;
+                  }
+                }
+                break;
+                case STARTTLS:
+                case MAYSTARTTLS: {
+                  char *remotename = NULL;
+                  int esmtp_options = 0;
+                  // fetch greeting
+                  if (SM_OK != SMTP_ok(ctl->smtp_socket, 'S', 0)) {
+smtp_starttls_failed_report:
+                    report(stderr, "%s: %s\n", idp->id,
+                           GT_("SMTP server greeted with error and does not want our mail.\n"));
+smtp_starttls_failed_silent:
+                    smtp_close(ctl, 0);
+                    xfree(parsed_host);
+                    continue;
+                  }
+                  int ok = SMTP_ehlo(ctl->smtp_socket, 'S', id_me, NULL,
+                                     NULL, &esmtp_options);
+                  if (!(SM_OK == ok && esmtp_options & ESMTP_STARTTLS)) {
+                    if (sinkdata.tlsmode != MAYSTARTTLS) goto smtp_starttls_failed_report; else goto no_opportunistic_starttls;
+                  }
+                  if (outlevel >= O_MONITOR)
+	            report(stdout, "SMTP> STARTTLS\n");
+                  SockPrintf(ctl->smtp_socket, "STARTTLS\r\n");
+                  if (SM_OK != SMTP_ok(ctl->smtp_socket, 'S', 0)) {
+                    if (MAYSTARTTLS == sinkdata.tlsmode) goto no_opportunistic_starttls;
+smtp_starttls_nego_failed:
+                    report(stderr, GT_("%s: upgrade to TLS failed.\n"), idp->id);
+                    goto smtp_starttls_failed_silent;
+                  }
+                  res = SSLOpen(ctl->smtp_socket, NULL, NULL, sinkdata.tlsproto, sinkdata.tlsmode != MAYSTARTTLS,
+                                    ctl->sslcertfile, ctl->sslcertpath, fingerprintmode,
+                                    sinkdata.servername, sinkdata.identifier, &remotename);
+                  if (res) {
+                    // OK - we opportunistically tried STARTTLS but it failed to negotiate TLS
+                    // now we're bust and must close the socket.
+                    if (sinkdata.tlsmode == MAYSTARTTLS) {
+                        report(stderr, GT_("%s: opportunistic upgrade to TLS failed, cannot continue.\n"
+                        "You can add /notls to the SMTP server name to avoid STARTTLS.\n"), idp->id);
+                        goto smtp_starttls_failed_silent;
+                    }
+                    goto smtp_starttls_nego_failed;
+                  }
+                  if (outlevel >= O_VERBOSE) {
+                        report(stdout, GT_("%s: upgrade to TLS succeeded.\n"), idp->id);
+                  }
+no_opportunistic_starttls:
+                  greeting_consumed = true;
+                }
+                break;
+#endif
+                case NOTLS:
+                break;
+                }
+            }
+
+            /* return immediately for ODMR */
 	    if (ctl->server.protocol == P_ODMR)
 	    {
 		set_timeout(0);
@@ -159,7 +301,7 @@ int smtp_setup(struct query *ctl)
 	    }
 
 	    /* first, probe for ESMTP */
-	    if (SMTP_ok(ctl->smtp_socket, ctl->smtphostmode, TIMEOUT_STARTSMTP) == SM_OK &&
+	    if ((greeting_consumed || SMTP_ok(ctl->smtp_socket, ctl->smtphostmode, TIMEOUT_STARTSMTP) == SM_OK) &&
 		    SMTP_ehlo(ctl->smtp_socket, ctl->smtphostmode, id_me,
 			ctl->server.esmtp_name, ctl->server.esmtp_password,
 			&ctl->server.esmtp_options) == SM_OK)
@@ -179,7 +321,17 @@ int smtp_setup(struct query *ctl)
 	    }
 	    else
 	    {
-		if ((ctl->smtp_socket = SockOpen(parsed_host,portnum,
+		if (ctl->server.esmtp_name || ctl->server.esmtp_password)
+		{
+		    // we have an ESMTP user name or password, but
+		    // old SMTP servers that don't understand EHLO
+		    // cannot negotiate AUTH SASL mechanisms, so we need
+		    // to skip
+		    xfree(parsed_host);
+		    continue;
+		}
+
+		if ((ctl->smtp_socket = SockOpen(parsed_host,sinkdata.portname,
 				ctl->server.plugout, &ai1)) == -1)
 		{
 		    xfree(parsed_host);
@@ -639,7 +791,7 @@ int stuffline(struct query *ctl, char *buf)
      */
     if (*buf == '.')
     {
-	if (ctl->server.base_protocol->delimited)	/* server has already byte-stuffed */
+        if (ctl->server.base_protocol->delimited)	/* server has already byte-stuffed */
 	{
 	    if (ctl->mda) {
 		/* writing to MDA, undo byte-stuffing */
@@ -1258,32 +1410,21 @@ int open_sink(struct query *ctl, struct msgblk *msg,
 
     if (ctl->bsmtp)		/* dump to a BSMTP batch file */
 	return(open_bsmtp_sink(ctl, msg, good_addresses, bad_addresses));
-    /* 
-     * Try to forward to an SMTP or LMTP listener.  If the attempt to 
-     * open a socket fails, fall through to attempt delivery via
-     * local MDA.
-     */
-    else if (!ctl->mda && smtp_setup(ctl) != -1)
-	return(open_smtp_sink(ctl, msg, good_addresses, bad_addresses));
-
-    /*
-     * Awkward case.  User didn't specify an MDA.  Our attempt to get a
-     * listener socket failed.  Try to cope anyway -- initial configuration
-     * may have found procmail.
-     */
-    else if (!ctl->mda)
-    {
-	report(stderr, GT_("%cMTP connect to %s failed\n"),
-	       ctl->smtphostmode,
-	       ctl->smtphost ? ctl->smtphost : "localhost");
-
-	return(PS_SMTP);
-    }
 
     if (ctl->mda)		/* must deliver through an MDA */
 	return(open_mda_sink(ctl, msg, good_addresses, bad_addresses));
 
-    return(PS_SUCCESS);
+        /* 
+     * Try to forward to an SMTP or LMTP listener.  If the attempt to 
+     * open a socket fails, fall through to attempt delivery via
+     * local MDA.
+     */
+    if (smtp_setup(ctl) != -1)
+	return(open_smtp_sink(ctl, msg, good_addresses, bad_addresses));
+
+    report(stderr, GT_("%cMTP connect to %s failed\n"), ctl->smtphostmode,
+           ctl->smtphost ? ctl->smtphost : "localhost");
+    return PS_SMTP;
 }
 
 void release_sink(struct query *ctl)
