@@ -52,6 +52,9 @@ static struct opt extensions[] =
 #ifdef ODMR_ENABLE
     {"ATRN",		ESMTP_ATRN},
 #endif /* ODMR_ENABLE */
+#ifdef SSL_ENABLE
+    {"STARTTLS",        ESMTP_STARTTLS},
+#endif
     {(char *)NULL, 0},
 };
 
@@ -271,7 +274,7 @@ static int SMTP_auth(int sock, char smtp_mode, const char *username, const char
 
 int SMTP_ehlo(int sock, char smtp_mode /**< 'S' for ESMTP or 'L' for LMTP */,
         const char *host, const char *name, const char *password, int *opt)
-/** send a "EHLO" or "LHLO" message to the SMTP or LMTP listener, return extension status bits in \a opt */
+/** send a "EHLO" or "LHLO" message to the SMTP or LMTP listener (depending on \a smtp_mode) via \a sock identifying ourselves as \a host, potentially log in with \a name and \a password through SMTP AUTH, and return extension status bits in \a *opt */
 {
   char auth_response[511];
   SIGHANDLERTYPE alrmsave;
@@ -308,8 +311,18 @@ int SMTP_ehlo(int sock, char smtp_mode /**< 'S' for ESMTP or 'L' for LMTP */,
 		strlcpy(auth_response, smtp_response, sizeof(auth_response));
 	  }
       if ((smtp_response[0] == '1' || smtp_response[0] == '2' || smtp_response[0] == '3') && smtp_response[3] == ' ') {
-	  if (*opt & ESMTP_AUTH && name && password) /* name is defaulted to the login name... */
-		return SMTP_auth(sock, smtp_mode, name, password, auth_response);
+          if (name && password) {
+            if (*opt & ESMTP_AUTH) {
+		if (SM_OK != SMTP_auth(sock, smtp_mode, name, password, auth_response)) {
+                        report_build(stderr, GT_("fetchmail authentication failed on %s@%s"), name, host);
+                        report_complete(stderr, "\n");
+                        return SM_UNRECOVERABLE;
+                }
+            } else {
+                report(stderr, "%cMTP: %s\n", smtp_mode, GT_("authentication requested, but not offered by server."));
+                return SM_UNRECOVERABLE;
+            }
+          }
 	  return SM_OK;
       }
       else if (smtp_response[3] != '-')
@@ -359,7 +372,7 @@ static char *SMTP_fixup_address_literal(const char *addr /** input address (user
 
     char newlit[INET_ADDRSTRLEN + 1] = "";
 
-    if (1 != match_regex("@[0-9a-fA-Fx:.]+[[:space:]]*$", addr))
+    if (1 != regex_ere_search("@[0-9a-fA-Fx:.]+[[:space:]]*$", addr))
         return xstrdup(addr);
 
     pos = strrchr(addr, '@');
