@@ -73,7 +73,7 @@ int UnixOpen(const char *path);
 
 #ifdef SSL_ENABLE
 int SSLOpen(int sock, char *mycert, char *mykey, const char *myproto, int certck, char *cacertfile, char *cacertpath,
-    char *fingerprint, char *servercname, char *label, char **remotename);
+    const char *fingerprint, char *servercname, char *label, char **remotename);
 #endif /* SSL_ENABLE */
 
 #endif /* SOCKET__ */
